/*
 *  radiusplugin -- An OpenVPN plugin for do radius authentication 
 *					and accounting.
 * 
 *  Copyright (C) 2005 EWE TEL GmbH/Ralf Luebben <ralfluebben@gmx.de>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
#include "AccountingProcess.h"

/** This method is the background process for accounting. It is in a endless loop
 * until it gets a exit command. In the loop the process is 
 * waiting for a command from the foregroundprocess (USER_ADD, USER_DEL, EXIT).
 * If no command is arrived in an interval of 0,5s the accounting is done
 * for all users who need a update. The interval is 0,5s because every second
 * a user can connect with an unknown interval, so this interval must be shorter.
 * @param context The plugin context as object from the class PluginContext.
 */

void AccountingProcess::Accounting(PluginContext * context)
{
	UserAcct				*user=NULL; //The user for acconting.
  	int 					command, 	//The command from foreground process.
  							result; 	//The result from the socket.
  	string 					commonname; //The commonname.
  	AcctScheduler 			scheduler; 	//The scheduler for the accounting.
  	fd_set  				set; 		//A set for the select function.
  	struct timeval 			tv; 		//A timeinterval for the select funtion.
  
  	
  		
	
	//Tell the parent everythink is ok.
  	try
  	{
  		context->acctsocketforegr.send(RESPONSE_INIT_SUCCEEDED);
  	}
  	catch (Exception &e)
    {
     	cerr << "RADIUS-PLUGIN: BACKGROUND-ACCT:" << e <<"\n";
      	goto done;
    }
    
   	if (DEBUG (context->getVerbosity()))
    	cerr << "RADIUS-PLUGIN: BACKGROUND ACCT: Started, RESPONSE_INIT_SUCCEEDED was sent to Foreground Process.\n";
   	   	
   	
   	// Event loop
  	while (1)
    {
    	//create the informations for the result function	
    	tv.tv_sec = 0;  
		tv.tv_usec = 500000; 	//wait 0,5s
		FD_ZERO(&set);			// clear out the set 
		FD_SET(context->acctsocketforegr.getSocket(), &set);		// wait only on the socket from the foreground process
   		result = select(FD_SETSIZE, &set, NULL, NULL, &tv);
		
		//if there is a data on the socket
  		if (result>0)
		{
      		// get a command from foreground process 
      		command = context->acctsocketforegr.recvInt();
      		
      		if (DEBUG (context->getVerbosity()))
    			cerr << "RADIUS-PLUGIN: BACKGROUND ACCT: Get a command.\n";
   
		    switch (command)
			{
			//add a new user to the scheduler
			case ADD_USER:
				try
				{
				
					if (DEBUG (context->getVerbosity()))
				    	cerr << "RADIUS-PLUGIN: BACKGROUND ACCT: New User.\n";
					
					//allocate memory				
					user= new UserAcct;
					
					//get the information from the foreground process
					user->setUsername(context->acctsocketforegr.recvStr());
					user->setSessionId(context->acctsocketforegr.recvStr()) ;
					user->setPortnumber(context->acctsocketforegr.recvInt()); 
					user->setCallingStationId(context->acctsocketforegr.recvStr()); 
					user->setFramedIp(context->acctsocketforegr.recvStr()); 
					user->setCommonname(context->acctsocketforegr.recvStr());
					user->setAcctInterimInterval(context->acctsocketforegr.recvInt());
					user->setFramedRoutes(context->acctsocketforegr.recvStr());
										
									
					if (DEBUG (context->getVerbosity()))
				    	cerr << "RADIUS-PLUGIN: BACKGROUND ACCT: New user acct: username: " << user->getUsername() << ", interval: " << user->getAcctInterimInterval() << ", calling station: " << user->getCallingStationId() << ", commonname: " << user->getCommonname() << ", framed ip: " << user->getFramedIp() <<".\n";
					
					
					//set the starttime
					user->setStarttime(time(NULL));
					
					//calculate the nextupdate
					user->setNextUpdate(user->getStarttime()+user->getAcctInterimInterval());
					
					//send the start packet
					if (user->sendStartPacket(context)==0)
					{
						
						if (DEBUG (context->getVerbosity()))
				    		cerr << "RADIUS-PLUGIN: BACKGROUND ACCT: Start packet was send.\n";
					
						//add the user to the scheduler
						scheduler.addUser(user);
						
							
						if (DEBUG (context->getVerbosity()))
				    		cerr << "RADIUS-PLUGIN: BACKGROUND ACCT: User was added to accounting scheduler.\n";
							
						//set the system routes
						user->addSystemRoutes(context);
					    
					    //send the ok to the parent process
						context->acctsocketforegr.send(RESPONSE_SUCCEEDED);								  		
						
					}
					else
					{
						
						//tell the parent parent process something is wrong
						throw Exception("Start packet couldn't send.\n");
						
					}
					// free the user, he was copied to the accounting scheduler list
					
				}
				catch (Exception &e)
				{
					cerr << "RADIUS-PLUGIN: BACKGROUND ACCT: "<< e << "!\n";
					context->acctsocketforegr.send(RESPONSE_FAILED);
					//close the background process, if the ipc socket is bad
					if (e.getErrnum()==Exception::SOCKETSEND || e.getErrnum()==Exception::SOCKETRECV)
					{
						goto done;
					}
				}
				catch (...)
				{
					context->acctsocketforegr.send(RESPONSE_FAILED);
					cerr << "RADIUS-PLUGIN: BACKGROUND ACCT: Unknown Exception!\n";
				}
				delete user;
				break;
			
			//delete a user	
			case DEL_USER:
				
				if (DEBUG (context->getVerbosity()))
			    	cerr << "RADIUS-PLUGIN: BACKGROUND-ACCT: Delete user from accounting.\n";
								
				//receive the information			
				try
				{
					commonname=context->acctsocketforegr.recvStr();
				}
				catch(Exception &e)
				{
					cerr << "RADIUS-PLUGIN: BACKGROUND ACCT: "<< e << "!\n";
					//close the background process, if the ipc socket is bad
					if (e.getErrnum()==Exception::SOCKETSEND || e.getErrnum()==Exception::SOCKETRECV)
					{
						goto done;
					}
				}
				catch (...)
				{
					cerr << "RADIUS-PLUGIN: BACKGROUND ACCT: Unknown Exception!\n";
				}	
				
				//find the user, he must be already there
				user=scheduler.findUser(commonname.c_str());
				
				if (user)
				{
					if (DEBUG (context->getVerbosity()))
				    	cerr << "RADIUS-PLUGIN: BACKGROUND ACCT: Stop acct: username: " << user->getUsername()<< ", calling station: " << user->getCallingStationId()<< ", commonname: " << user->getCommonname() << ".\n";
					
					//delete the system routes
					user->delSystemRoutes(context);
					
					try
					{
						//delete the user from the accounting scheduler
						scheduler.delUser(context, user);
						
						if (DEBUG (context->getVerbosity()))
				    		cerr << "RADIUS-PLUGIN: BACKGROUND ACCT: User with commonname " << commonname << " was deleted from accouting.\n";
													
						//send the parent process the ok
						context->acctsocketforegr.send(RESPONSE_SUCCEEDED);
						
						
					}
					catch(Exception &e)
					{
						cerr << "RADIUS-PLUGIN: BACKGROUND-ACCT: " << e << "\n";
						goto done;
					}
					catch (...)
					{
						cerr << "RADIUS-PLUGIN: BACKGROUND ACCT: Unknown Exception!\n";
					}	
				}
				else
				{
					cerr << "RADIUS-PLUGIN: BACKGROUND ACCT: No user with this commonname "<< commonname <<".\n";
					context->acctsocketforegr.send(RESPONSE_FAILED);
						
				}
				break;
				
			//exit the loop
			case COMMAND_EXIT:
				if (DEBUG (context->getVerbosity()))
		    		cerr << "RADIUS-PLUGIN: BACKGROUND-ACCT: Get command exit.\n";
	  			goto done;
	  		
	  		case -1:
				  cerr << "RADIUS-PLUGIN: BACKGROUND: read error on command channel.\n";
				  break;

			default:
			  cerr << "RADIUS-PLUGIN: BACKGROUND: unknown command code: code= "<< command <<", exiting.\n";
			  goto done;
			
		
			}
		}
		//after 0,5sec without a command call the scheduler
		scheduler.doAccounting(context);
		
    }
	done:
		//end the process
 		if(1)
 		scheduler.delallUsers(context);
 		cerr << "RADIUS-PLUGIN: BACKGROUND ACCT: EXIT\n";
  		return;
}

